@@ -12,7 +12,7 @@
 from __future__ import absolute_import, unicode_literals
 import sys
 
-from .client import LineClient
+from .client import LineClient, LineGroup, LineContact
 
-__version__ = '0.0.7'
-__all__ = ['LineClient']
+__version__ = '0.0.8'
+__all__ = ['LineClient','LineGroup','LineContact']
