@@ -26,6 +26,7 @@ sys.setdefaultencoding("utf-8")
 #from curve import CurveThrift
 from curve import CurveThrift
 from curve.ttypes import TalkException
+from curve.ttypes import ToType, ContentType
 
 try:
     import simplejson as json
@@ -39,62 +40,36 @@ EMAIL_REGEX = re.compile(r"[^@]+@[^@]+\.[^@]+")
 class LineMessage:
     """LineMessage wrapper"""
 
-    TEXT      = 0
-    IMAGE     = 1
-    VIDEO     = 2
-    AUDIO     = 3
-    HTML      = 4
-    PDF       = 5
-    CALL      = 6
-    STICKER   = 7
-    PRESENCE  = 8
-    GIFT      = 9
-    GROUBOARD = 10
-    APPLINK   = 11
-
     def __init__(self, client, message):
         self._client = client
-        self.type = message.type
-        self.text = message.text
         self.id   = message.id
+        self.text = message.text
 
-        self.sender   = client.getContactFromId(message._from)
-        self.receiver = client.getContactFromId(message.to)
-
-        self.sendTime = datetime.fromtimestamp(message.createdTime/1000)
+        self.hasContent = message.hasContent
+        self.contentType = message.contentType
         self.contentPreview = message.contentPreview
+        self.contentMetadata = message.contentMetadata
+
+        self.sender   = client.getContactOrGroupFromId(message._from)
+        self.receiver = client.getContactOrGroupFromId(message.to)
+
+        # toType
+        # 0: User
+        # 1: Room
+        # 2: Group
+        self.toType = message.toType
+        self.createdTime = datetime.fromtimestamp(message.createdTime/1000)
 
     def __repr__(self):
-        return 'LineMessage (type=%s, sender=%s, receiver=%s) "%s"' % (
-                    self.type,
+        return 'LineMessage (contentType=%s, sender=%s, receiver=%s) "%s"' % (
+                    ContentType._VALUES_TO_NAMES[self.contentType],
+                    #ToType._VALUES_TO_NAMES[self.toType],
                     self.sender,
                     self.receiver,
                     self.text
                 )
 
-class LineGroup(object):
-    """LineGroup wrapper
-    
-    :param client: LineClient instance
-    :param group: Group instace
-    """
-    def __init__(self, client, group):
-        self._client  = client
-        self._group   = group
-
-        self.id      = group.id
-        self.name    = group.name
-        self.creator = LineContact(client, group.creator)
-
-        self.members = []
-        for member in group.members:
-            self.members.append(LineContact(client, member))
-
-        self.invitee = []
-        if group.invitee:
-            for member in group.invitee:
-                self.invitee.append(LineContact(client, member))
-
+class LineBase(object):
     def sendMessage(self, text):
         try:
             message = CurveThrift.Message(to=self.id, text=text)
@@ -105,73 +80,7 @@ class LineGroup(object):
             raise e
 
     def sendSticker(self,
-                    stickerId = "1",
-                    stickerPackageId = "1",
-                    stickerVersion = "100",
-                    stickerText="[null]"):
-        try:
-            message = CurveThrift.Message(to=self.id, text="")
-            message.contentType = CurveThrift.ContentType.STICKER
-
-            message.contentMetadata = {
-                'STKID': stickerId,
-                'STKPKGID': stickerPackageId,
-                'STKVER': stickerVersion,
-                'STKTXT': stickerText,
-            }
-
-            self._client._sendMessage(message)
-
-            return True
-        except Exception as e:
-            raise e
-
-    def getRecentMessages(self, count=1):
-        """Get recent messages"""
-        try:
-            messages = self._client._getRecentMessages(self.messageBox.id, count)
-            return messages
-        except:
-            messageBoxWrapUp = self._client._getMessageBoxCompactWrapUp(self.id)
-            self.messageBox = messageBoxWrapUp.messageBox
-
-            messages = self._client._getRecentMessages(self.messageBox.id, count)
-            return messages
-
-    def __repr__(self):
-        """Name of Group and number of group members"""
-        return '<LineGroup %s #%s>' % (self.name, len(self.members))
-
-    def __lt__(self, other):
-        return self.id < other.id
-
-class LineContact(object):
-    """LineContact wrapper
-    
-    :param client: LineClient instance
-    :param contact: Conatct instace
-    """
-    def __init__(self, client, contact):
-        self._client  = client
-        self._contact = contact
-
-        self.id           = contact.mid
-        self.displayName   = contact.displayName
-        self.statusMessage = contact.statusMessage
-
-    def getRecentMessages(self, count=1):
-        try:
-            messages = self._client._getRecentMessages(self.messageBox.id, count)
-            return messages
-        except:
-            messageBoxWrapUp = self._client._getMessageBoxCompactWrapUp(self.id)
-            self.messageBox = messageBoxWrapUp.messageBox
-
-            messages = self._client._getRecentMessages(self.messageBox.id, count)
-            return messages
-
-    def sendSticker(self,
-                    stickerId = "1",
+                    stickerId = "13",
                     stickerPackageId = "1",
                     stickerVersion = "100",
                     stickerText="[null]"):
@@ -237,26 +146,108 @@ class LineContact(object):
         except Exception as e:
             raise e
 
-    def sendMessage(self, text):
+    def _getLineMessageListFromMessageList(self, messages=[]):
+        message_list = []
+        for message in messages:
+            message_list.append(LineMessage(self._client, message))
+
+        return message_list
+
+    def getRecentMessages(self, count=1):
+        """Get recent messages"""
         try:
-            message = CurveThrift.Message(to=self.id, text=text)
-            self._client._sendMessage(message)
+            messages = self._client._getRecentMessages(self.messageBox.id, count)
 
-            return True
-        except Exception as e:
-            raise e
+            return self._getLineMessageListFromMessageList(messages)
+        except:
+            self.messageBox = self._client.getMessageBox(self.id)
 
-    def __repr__(self):
-        return '<LineContact %s (%s)>' % (self.displayName, self.id)
+            messages = self._client._getRecentMessages(self.messageBox.id, count)
+
+            return self._getLineMessageListFromMessageList(messages)
 
     def __lt__(self, other):
         return self.id < other.id
 
+class LineGroup(LineBase):
+    """LineGroup wrapper
+    
+    Attributes:
+        creator     contact of group creator
+        members     list of contact of group members
+        invitee     list of contact of group invitee
+    """
+    def __init__(self, client, group):
+        """LineGroup init
+
+        :param client: LineClient instance
+        :param group: Group instace
+        """
+
+        self._client  = client
+        self._group   = group
+
+        self.id      = group.id
+        self.name    = group.name
+
+        try:
+            self.creator = LineContact(client, group.creator)
+        except:
+            self.creator = None
+
+        self.members = []
+        for member in group.members:
+            self.members.append(LineContact(client, member))
+
+        self.invitee = []
+        if group.invitee:
+            for member in group.invitee:
+                self.invitee.append(LineContact(client, member))
+
+    def leaveGroup(self):
+        """Leave group"""
+        try:
+            self._client._leaveGroup(self.id)
+            return True
+        except:
+            return False
+
+    def __repr__(self):
+        """Name of Group and number of group members"""
+        return '<LineGroup %s #%s>' % (self.name, len(self.members))
+
+class LineContact(LineBase):
+    """LineContact wrapper
+    
+    Attributes:
+        name            display name of contact
+        statusMessage   status message of contact
+    """
+    def __init__(self, client, contact):
+        """LineContact init
+
+        :param client: LineClient instance
+        :param contact: Conatct instace
+        """
+
+        self._client  = client
+        self._contact = contact
+
+        self.id            = contact.mid
+        self.name          = contact.displayName
+        self.statusMessage = contact.statusMessage
+
+    def __repr__(self):
+        #return '<LineContact %s (%s)>' % (self.name, self.id)
+        return '<LineContact %s>' % (self.name)
+
 class LineClient(object):
     """This class proviede you a way to communicate with LINE server.
 
         >>> client = LineClient(\'carpedm20\', \'xxxxxxxxxx\')
         Enter PinCode '9779' to your mobile phone in 2 minutes
+        >>> client = LineClient(\'carpedm20@gmail.com\', \'xxxxxxxxxx\')
+        Enter PinCode '' to your mobile phone in 2 minutes
         >>> print client.profile
     """
 
@@ -345,8 +336,8 @@ class LineClient(object):
         self.protocol    = TCompactProtocol.TCompactProtocol(self.transport)
         self.protocol_in = TCompactProtocol.TCompactProtocol(self.transport_in)
 
-        self.client    = CurveThrift.Client(self.protocol)
-        self.client_in = CurveThrift.Client(self.protocol_in)
+        self._client    = CurveThrift.Client(self.protocol)
+        self._client_in = CurveThrift.Client(self.protocol_in)
 
         self.transport.open()
         self.transport_in.open()
@@ -371,9 +362,9 @@ class LineClient(object):
         self.transport.setCustomHeaders(self._headers)
 
         self.protocol = TCompactProtocol.TCompactProtocol(self.transport)
-        self.client   = CurveThrift.Client(self.protocol)
+        self._client   = CurveThrift.Client(self.protocol)
 
-        msg = self.client.loginWithIdentityCredentialForCertificate(
+        msg = self._client.loginWithIdentityCredentialForCertificate(
                 self.id, self.password, keyname, crypto, False, self.ip,
                 self.com_name, self.provider, "")
         
@@ -386,7 +377,7 @@ class LineClient(object):
         j = self.get_json(self.LINE_CERTIFICATE_URL)
         self.verifier = j['result']['verifier']
 
-        msg = self.client.loginWithVerifierForCertificate(self.verifier)
+        msg = self._client.loginWithVerifierForCertificate(self.verifier)
 
         if msg.type == 1:
             self.certificate = msg.certificate
@@ -411,6 +402,12 @@ class LineClient(object):
 
             self.groups.sort()
 
+    def refreshRooms(self):
+        """Refresh rooms. Need to be called after `refreshContacts`"""
+        if self.check_auth():
+            for contact in self.contacts:
+                messageBox = self.getMessageBox(contact.id)
+
     def refreshContacts(self):
         """Refresh contacts of LineClient """
         if self.check_auth():
@@ -434,6 +431,20 @@ class LineClient(object):
 
         return None
 
+    def getGroupFromName(self, name):
+        for group in self.groups:
+            if name == group.name:
+                return group
+
+        return None
+
+    def getContactFromName(self, name):
+        for contact in self.contacts:
+            if name == contact.name:
+                return contact
+
+        return None
+
     def getGroupFromId(self, id):
         for group in self.groups:
             if group.id == id:
@@ -452,11 +463,27 @@ class LineClient(object):
 
         return None
 
-    def longPoll(self, count=10, func=None):
+    def getMessageBox(self, id):
+        try:
+            messageBoxWrapUp = self._client._getMessageBoxCompactWrapUp(self.id)
+
+            return messageBoxWrapUp.messageBox
+        except:
+            return None
+
+    def longPoll(self, count=50):
         """Check is there any operations from LINE server"""
         OT = CurveThrift.OperationType
 
-        operations = self.client_in.fetchOperations(self.revision, count)
+        try:
+            operations = self._client_in.fetchOperations(self.revision, count)
+        except EOFError:
+            return
+        except TalkException as e:
+            if e.code == 9:
+                raise_error("user logged in to another machien")
+            else:
+                return
 
         for operation in operations:
             if operation.type == OT.END_OF_OPERATION:
@@ -464,14 +491,12 @@ class LineClient(object):
             elif operation.type == OT.SEND_MESSAGE:
                 pass
             elif operation.type == OT.RECEIVE_MESSAGE:
-                message = operation.message
+                message = LineMessage(self, operation.message)
                 group_or_contact = self.getContactOrGroupFromId(message.to)
 
-                if func != None:
-                    func(group_or_contact, message)
-
                 yield (group_or_contact, message)
             else:
+                print "[*] %s" % OT._VALUES_TO_NAMES[operation.type]
                 print operation
 
             self.revision = max(operation.revision, self.revision)
@@ -510,12 +535,12 @@ class LineClient(object):
                     - statusMessage
         """
         if self.check_auth():
-            return self.client.getProfile()
+            return self._client.getProfile()
 
     def _getAllContactIds(self):
         """Get all contacts of your LINE account"""
         if self.check_auth():
-            return self.client.getAllContactIds()
+            return self._client.getAllContactIds()
 
     def _getContacts(self, ids):
         """Get contact information list from ids
@@ -545,12 +570,12 @@ class LineClient(object):
             raise_error(msg)
 
         if self.check_auth():
-            return self.client.getContacts(ids)
+            return self._client.getContacts(ids)
 
     def _getGroupIdsJoined(self):
         """Get group id that you joined"""
         if self.check_auth():
-            return self.client.getGroupIdsJoined()
+            return self._client.getGroupIdsJoined()
 
     def _getGroups(self, ids):
         if type(ids) != list:
@@ -558,12 +583,17 @@ class LineClient(object):
             raise_error(msg)
 
         if self.check_auth():
-            return self.client.getGroups(ids)
+            return self._client.getGroups(ids)
+
+    def _leaveGroup(self, id):
+        """Leave a group"""
+        if self.check_auth():
+            return self._client.leaveGroup(0, id)
 
     def _getRecentMessages(self, id, count=1):
         """Get recent messages from `id`"""
         if self.check_auth():
-            return self.client.getRecentMessages(id, count)
+            return self._client.getRecentMessages(id, count)
 
     def _sendMessage(self, message, seq=0):
         """Send a message to `id`. `id` could be contact id or group id
@@ -572,20 +602,20 @@ class LineClient(object):
         :param message: `message` instance
         """
         if self.check_auth():
-            return self.client.sendMessage(seq, message)
+            return self._client.sendMessage(seq, message)
 
     def _getLastOpRevision(self):
         if self.check_auth():
-            return self.client.getLastOpRevision()
+            return self._client.getLastOpRevision()
 
     def _fetchOperations(self, revision, count=50):
         if self.check_auth():
-            return self.client.fetchOperations(revision, count)
+            return self._client.fetchOperations(revision, count)
 
     def _getMessageBoxCompactWrapUp(self, id):
         if self.check_auth():
             try:
-                return self.client.getMessageBoxCompactWrapUp(id)
+                return self._client.getMessageBoxCompactWrapUp(id)
             except Exception as e:
                 msg = e
                 raise_error(msg)
